quit_msgs = [
        "Deus abençoe, vá em paz",
        "Bye bye",
        "Vou sentir saudades",
        "Volte quando estiver mais descansado",
        "Espero que tenha aprendido algo novo hoje",
        "Cada pequena vitória merece ser comemorada",
        "Obrigado pelo tempo juntos",
        "Até a próxima",
        "Um abraço, seja feliz", 
        "Durma cedo, amanhã você pode continuar", 
        "Vá, mais volte",
        "Arri égua, já vai?", 
        "Capando o gato tão cedo?",
        "Vai porque quer",
        "Fique mais um pouquinho", 
        "Vamos lá, pegue um café (ou chá) e volte outra hora",
        "Foque em avançar um passo por vez",
        "O caminho é mais importante que a chegada",
        "Lembre de dar bons nomes para as variáveis e métodos", 
        "Você também está programando para outros seres humanos lerem, faça algo organizado",
        "Seja criterioso com a indentação e as regras de estilo, faça algo belo",
        "Está precisando de mais conselhos? Experimente rezar",
        "Não se repita, refatore seu código para tirar as redundâncias",
        "São necessárias umas 1000 horas pra ser bom em algo",
        "Só é fracasso quando você desiste, espaireça e tente de novo",
        "Parar e descansar é importante também",
        "Se for pra descansar, largue o celular",
        "Traga uma garrafa de água próxima vez, facilite o acesso",
        "Toda capacidade que não é usada, se perde, movimente-se",
        "Te vejo depois do exercício. Malhe, corra, dance, e depois volte",
        "Se você não se der valor, ninguém vai. Invista em você",
        "Tem exercícios que vão ser bem complicados, tire um tempo para pensar longe do computador",
        "Use as inteligências artificiais com sabedoria, depois, tente refazer a tarefa sem elas",
        "Lembre de pedir ajuda quando precisar", 
        "Você não está sozinho, peça ajuda se preciso", 
        "É justo que o que muito vale, muito custe", 
        "Aprender a se divertir é parte essencial do aprendizado", 
        "Esse miserável é um miserável",
        "Você é importante e precioso, mesmo que seu código tenha ficado uma porcaria",
        "Alguns tem mais dificuldades que outros, tenha paciência consigo mesmo",
        "Se você precisa de mais tempo que seus colegas pra aprender, se dê mais tempo", 
    ]